@@ -181,9 +181,9 @@ sub BuildDSN {
     Returns the DSN for this database connection.
 
 =cut
+
 sub DSN {
-    my $self = shift;
-    return($self->{'dsn'});
+    return shift->{'dsn'};
 }
 
 
@@ -252,7 +252,7 @@ sub _LogSQLStatement {
     my $statement = shift;
     my $duration = shift;
     my @bind = @_;
-    push @{$self->{'StatementLog'}} , ([Time::HiRes::time(), $statement, [@bind], $duration]);
+    push @{$self->{'StatementLog'}} , ([Time::HiRes::time(), $statement, [@bind], $duration, Carp::longmess("Executed SQL query")]);
 
 }
 
@@ -672,13 +672,16 @@ Returns a FIELD OPERATOR VALUE triple.
 
 =cut
 
+our $RE_CASE_INSENSITIVE_CHARS = qr/[-'"\d: ]/;
+
 sub _MakeClauseCaseInsensitive {
     my $self = shift;
     my $field = shift;
     my $operator = shift;
     my $value = shift;
 
-    if ($value !~ /^\d+$/) { # don't downcase integer values
+    # don't downcase integer values and things that looks like dates
+    if ($value !~ /^$RE_CASE_INSENSITIVE_CHARS+$/o) {
         $field = "lower($field)";
         $value = lc($value);
     }
@@ -1085,9 +1088,11 @@ sub _BuildJoins {
 
     # get a @list of joins that have not been processed yet, but depend on processed join
     my $joins = $sb->{'left_joins'};
-    while ( my @list = grep !$processed{ $_ }
-            && $processed{ $joins->{ $_ }{'depends_on'} }, keys %$joins )
-    {
+    while ( my @list =
+        grep !$processed{ $_ }
+            && (!$joins->{ $_ }{'depends_on'} || $processed{ $joins->{ $_ }{'depends_on'} }),
+        keys %$joins
+    ) {
         foreach my $join ( @list ) {
             $processed{ $join }++;
 
@@ -1196,10 +1201,15 @@ sub MayBeNull {
     foreach ( splice @conditions ) {
         unless ( ref $_ ) {
             push @conditions, $_;
-        } elsif ( $_->{'field'} =~ /^\Q$args{'ALIAS'}./ ) {
+        } elsif ( rindex( $_->{'field'}, "$args{'ALIAS'}.", 0 ) == 0 ) {
+            # field is alias.xxx op ... and only IS op allows NULLs
             push @conditions, lc $_->{op} eq 'is';
-        } elsif ( $_->{'value'} && $_->{'value'} =~ /^\Q$args{'ALIAS'}./ ) {
+        } elsif ( $_->{'value'} && rindex( $_->{'value'}, "$args{'ALIAS'}.", 0 ) == 0 ) {
+            # value is alias.xxx so it can not be IS op
             push @conditions, 0;
+        } elsif ( $_->{'field'} =~ /^(?i:lower)\(\s*\Q$args{'ALIAS'}\./ ) {
+            # handle 'LOWER(alias.xxx) OP VALUE' we use for case insensetive
+            push @conditions, lc $_->{op} eq 'is';
         } else {
             push @conditions, 1;
         }
@@ -1222,9 +1232,10 @@ sub MayBeNull {
     };
 
     # solve boolean expression we have, an answer is our result
+    my $parens_count = 0;
     my @tmp = ();
     while ( defined ( my $e = shift @conditions ) ) {
-        #warn "@tmp >>>$e<<< @conditions";
+        #print "@tmp >>>$e<<< @conditions\n";
         return $e if !@conditions && !@tmp;
 
         unless ( $e ) {
@@ -1242,7 +1253,7 @@ sub MayBeNull {
                 my $close_p = $closing_paren->(0);
                 splice @conditions, 0, $close_p + 1, (0);
             } else {
-                die "lost @tmp >>>$e $aggreg<<< @conditions";
+                die "unknown aggregator: @tmp $e >>>$aggreg<<< @conditions";
             }
         } elsif ( $e eq '1' ) {
             if ( $conditions[0] eq ')' ) {
@@ -1259,15 +1270,19 @@ sub MayBeNull {
                 # 1 AND x == x
                 next;
             } else {
-                die "lost @tmp >>>$e $aggreg<<< @conditions";
+                die "unknown aggregator: @tmp $e >>>$aggreg<<< @conditions";
             }
         } elsif ( $e eq '(' ) {
             if ( $conditions[1] eq ')' ) {
                 splice @conditions, 1, 1;
             } else {
+                $parens_count++;
                 push @tmp, $e;
             }
         } elsif ( $e eq ')' ) {
+            die "extra closing paren: @tmp >>>$e<<< @conditions"
+                if --$parens_count < 0;
+
             unshift @conditions, @tmp, $e;
             @tmp = ();
         } else {
